package org.q.app.config;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.MapTool;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

/**
 * Class responsible for loading scripts. Scripts must have a unique name, but will be found regardless of their
 * location as long as they are located in the scripts directory. This makes it possible to re-organize scripts in
 * directories without breaking functionality.
 * 
 * @author Erik Lievaart
 */
public class ScriptResources {

	@Inject
	private ConfigurableResources resources;
	private final ConcurrentMap<String, String> paths = NewCollection.concurrentMap();

	@Inject
	ScriptResources(final ApplicationResources application, final UserResources user) {
		try {
			List<String> applicationScripts = application.listAll(ResourcePaths.SCRIPT_DIR);
			List<String> userScripts = user.listAll(ResourcePaths.SCRIPT_DIR);
			paths.putAll(indexScripts(applicationScripts, userScripts));

		} catch (IOException e) {
			throw new RuntimeIOException("Unable to read resources in: " + ResourcePaths.SCRIPT_DIR, e);
		}
	}

	static Map<String, String> indexScripts(final List<String> application, final List<String> user) {
		return MapTool.merge(mapBaseNameToPath(application), mapBaseNameToPath(user));
	}

	static Map<String, String> mapBaseNameToPath(final List<String> applicationScripts) {
		Map<String, String> baseNameToPath = NewCollection.map();

		for (String path : applicationScripts) {
			String name = UrlTool.getBaseName(path);
			String current = baseNameToPath.get(name);
			Check.isNull(current, "Duplicate script %: % & %", name, current, path);
			baseNameToPath.put(name, path);
		}
		return baseNameToPath;
	}

	/**
	 * Returns the absolute path to a script. Depending on whether the script has been overridden, this may be an
	 * application resource or a user resource.
	 * 
	 * @param script
	 *            name of the script.
	 * @return the absolute location of the script.
	 * @throws IOException
	 *             if there was a problem performing IO.
	 */
	public String getAbsolutePath(final String script) throws IOException {
		return resources.getAbsolutePath(paths.get(script));
	}

	/**
	 * Check that a script exists with the specified name.
	 * 
	 * @param resource
	 *            name of the script.
	 * @return whether or not it exists.
	 */
	public boolean isAvailable(final String resource) {
		return paths.containsKey(resource);
	}
}
