package org.q.app.contract;

import org.q.toolkit.lang.Str;

/**
 * Exception used to signal that a ShellCommand's configuration is not valid.
 * 
 * @author Erik Lievaart
 */
public class ValidationException extends Exception {

	/**
	 * Constructor with message.
	 * 
	 * @param message
	 *            Message to the user why the command is not valid.
	 */
	public ValidationException(final String message) {
		super(message);
	}

	/**
	 * Constructor with message and cause.
	 * 
	 * @param message
	 *            Message to the user why the command is not valid.
	 * @param cause
	 *            Cause of the ValidationException.
	 */
	public ValidationException(final String message, final Throwable cause) {
		super(message, cause);
	}

	/**
	 * Shorthand for checking a boolean flag and throwing a ValidationException if the flag is true.
	 * 
	 * @param condition
	 *            throw a ValidationException if condition evaluates to true.
	 * @param format
	 *            Template for the message with '%' as place holders.
	 * @param args
	 *            substitution arguments for the '%' symbols in the format.
	 * @throws ValidationException
	 *             if condition evaluates to true.
	 */
	public static void on(final boolean condition, final String format, final Object... args)
			throws ValidationException {

		if (condition) {
			throw new ValidationException(Str.sub(format, args));
		}
	}

	/**
	 * Shorthand for checking a boolean flag and throwing a ValidationException if the flag is false.
	 * 
	 * @param condition
	 *            throw a ValidationException if condition evaluates to false.
	 * @param format
	 *            Template for the message with '%' as place holders.
	 * @param args
	 *            substitution arguments for the '%' symbols in the format.
	 * @throws ValidationException
	 *             if condition evaluates to false.
	 */
	public static void unless(final boolean condition, final String format, final Object... args)
			throws ValidationException {

		if (!condition) {
			throw new ValidationException(Str.sub(format, args));
		}
	}
}
