package org.q.app.engine.impl;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.q.app.engine.meta.CommandMetadata;
import org.q.app.engine.meta.FlagMetadata;
import org.q.app.engine.meta.MetadataCollection;
import org.q.app.engine.parse.ShellArgument;
import org.q.app.engine.parse.ShellCommand;
import org.q.app.engine.parse.ShellCommandMissingException;
import org.q.app.engine.parse.ShellFlagMissingException;
import org.q.app.engine.parse.ShellParser;
import org.q.app.general.DetailMessage;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

/**
 * The input examiner's main function is to provide feedback what a ShellCommand does if it is invoked.
 * 
 * @author Erik Lievaart
 */
public class InputExaminer {
	private final LogTemplate log = new LogTemplate(getClass());

	@Inject
	private MetadataCollection metadata;
	@Inject
	private EngineTemplates templates;
	@Inject
	private SafeOperations protection;
	@Inject
	private VariableExaminer variables;

	InputExaminer() {
	}

	/**
	 * Examine a ShellCommand after applying engine templates and return a detail or error message for the command.
	 * 
	 * @param raw
	 *            ShellCommand that needs an engine template applied.
	 * @return a {@link DetailMessage} indicating the details of the command that will be invoked. Default variables
	 *         will be added to the raw invocation and flags will be replaced with their readable counterparts. If the
	 *         command is not valid, an error message will be returned describing the cause of the problem.
	 */
	public DetailMessage examineTemplated(final String raw) {
		try {
			if (protection.isAllowed(templates.apply(raw))) {
				return examineRaw(templates.apply(raw));
			}
			return protection.getError();

		} catch (Exception e) { // Better catch this Exception, or user will be in endless dialog loop.
			log.warn("Unable to examine input", e);
			return DetailMessage.error("Unexpected error parsing input: " + e.getMessage());
		}

	}

	/**
	 * Examine a raw ShellCommand and return a detail or error message for the command.
	 * 
	 * @param raw
	 *            unparsed ShellCommand.
	 * @return a {@link DetailMessage} indicating the details of the command that will be invoked. Default variables
	 *         will be added to the raw invocation and flags will be replaced with their readable counterparts. If the
	 *         raw command is not valid, an error message will be returned describing the cause of the problem.
	 */
	public DetailMessage examineRaw(final String raw) {
		try {
			ShellCommand parsed = ShellParser.parseLine(raw);
			metadata.validate(parsed);
			variables.validate(parsed);
			return DetailMessage.message(toString(parsed));

		} catch (ShellCommandMissingException scm) {
			return DetailMessage.error("Commands: " + metadata.listCommands());

		} catch (ShellFlagMissingException sfm) {
			String cmd = sfm.getShellCommand().getName();
			if (cmd == null || !metadata.lookup(cmd).getName().getQName().startsWith(cmd)) {
				return DetailMessage.error(Str.sub("Cannot lookup flag; missing command %", cmd));
			}
			return DetailMessage.error(getFlagString(metadata.lookup(cmd)));

		} catch (Exception other) {
			return DetailMessage.error(other.getMessage());
		}
	}

	private String getFlagString(final CommandMetadata command) {
		StringBuilder builder = new StringBuilder(command.getName().getQName());
		builder.append(" -").append(Arrays.toString(command.getCharacterFlags())).append(" => ");

		List<String> flags = NewCollection.list();
		for (char f : command.getCharacterFlags()) {
			flags.add(command.getFlagMetadata(f).getName()); // does not swallow
		}
		return builder.append(flags).toString();
	}

	private String toString(final ShellCommand cmd) {
		CommandMetadata command = metadata.lookup(cmd.getName());
		ShellStringBuilder builder = new ShellStringBuilder(command.getName().getQName());

		Iterator<ShellArgument> iter = cmd.getArguments();
		for (char f : command.getFlagsOrDefaults(cmd)) {

			FlagMetadata flag = command.getFlagMetadata(f);
			builder.appendFlagName(flag.getName());

			for (ShellArgument argument : flag.getShellArguments(iter)) {
				builder.appendArgument(argument);
			}
		}
		return builder.appendSwallowed(cmd.getSwallowed()).toString();
	}
}