package org.q.app.vfs;

import java.io.File;

import org.apache.commons.vfs.FileObject;

/**
 * Convenience interface for working with the File system. The method signatures
 * of this interface closely resemble the {@link File} and the
 * {@link FileObject} method signatures to minimize confusion. All of the
 * methods on this interface can throw
 * {@link org.q.toolkit.io.RuntimeIOException}'s when invoked. Since these are
 * {@link RuntimeException}'s you are not required to handle them. URL's can be
 * represented in two ways: 1) human readable 2) escaped. For example the URL
 * "file:///c:/program files" would be human readable and the URL
 * "file:///program%20files" would be escaped. Depending on what the URL is used
 * for one of both formats sometimes won't work. The native command line
 * typically accepts human readable form, while ShellCommand's expect the
 * escaped form. Most properties on this class are available in both formats.
 * 
 * @author Erik Lievaart
 */
public interface VirtualFile {

	/**
	 * Returns true iff this VirtualFile is a directory.
	 */
	boolean isDirectory();

	/**
	 * Returns whether or not the resource exists.
	 */
	boolean exists();

	/**
	 * Returns true iff the resource if a file.
	 */
	boolean isFile();

	/**
	 * Returns true iff the resource is marked as hidden.
	 */
	boolean isHidden();

	/**
	 * Returns true iff the resource is a {@link File} on the local file system.
	 */
	boolean isLocal();

	/**
	 * Returns the escaped full URL.
	 */
	String getUrlEscaped();

	/**
	 * Returns the full URL in human readable form.
	 */
	String getUrl();

	/**
	 * Return the absolute path to the file.
	 */
	String getAbsolutePath();

	/**
	 * Get the base name of the file without the extension in human readable
	 * form.
	 */
	String getBaseName();

	/**
	 * Get the name of the file in human readable form including extension.
	 */
	String getName();

	/**
	 * Get the escaped name of the file including extension.
	 */
	String getNameEscaped();

	/**
	 * Get the file extension.
	 */
	String getExtension();

	/**
	 * Get the {@link File} for this VirtualFile. Use isLocal() to check if the
	 * VirtualFile is a File on the local file system, otherwise the result of
	 * this method is undefined.
	 */
	File getLocalFile();

	/**
	 * List the children of this VirtualFile in random order.
	 */
	VirtualFile[] listFiles();

	/**
	 * Return true iff this is not a root URL. The parent is not guaranteed to
	 * be accessible.
	 */
	public boolean hasParent();

	/**
	 * Get the parent file that has this file as a child.
	 */
	VirtualFile getParentFile();

	/**
	 * Get the Apache commons VFS {@link FileObject} for this VirtualFile.
	 */
	FileObject getFileObject();

	/**
	 * Resolve a child.
	 * 
	 * @param relative
	 *            Relative path to the child.
	 * @return the File if it could be resolved.
	 */
	VirtualFile resolveFile(String relative);

	/**
	 * Determines the size of the file, in bytes.
	 * 
	 * @return The size of the file, in bytes.
	 */
	long getSize();
}
