package org.q.plugins.ant;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.q.app.config.ConfigurableResources;
import org.q.app.contract.Invocable;
import org.q.app.contract.PluginContext;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.collection.MapTool;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.QUEUE)
@Doc("Call Ant build files to perform advanced file operations.")
class AntShellCommand implements Invocable {

	private static final String BUILD_FILE = "ant/q/app.xml";
	private static final String TO_PROPERTY = "q.to";
	private static final String FROM_POPERTY = "q.from";

	private final Log log = LogFactory.getLog(getClass());
	private final Map<String, String> properties = new Hashtable<String, String>();
	private String xmlFile = BUILD_FILE;
	private String target;

	@Inject
	private PluginContext plugin;
	@Inject
	private ConfigurableResources classpath;
	@Inject
	private BrowseEvents events;

	@Flag(group = "main", values = { "$dir", "$dir~" }, primary = true)
	@Doc("Copy a collection of files. Two arguments: 1) The source dir 2) The destination dir")
	public AntShellCommand copy(final File source, final File destination) {
		properties.put(FROM_POPERTY, source.getAbsolutePath());
		properties.put(TO_PROPERTY, destination.getAbsolutePath());
		target = "q.copy";
		return this;
	}

	@Flag(group = "main", values = { "$dir", "$dir~" })
	@Doc("Move a collection of files. Two arguments: 1) The source dir 2) The destination dir")
	public AntShellCommand move(final File source, final File destination) {
		properties.put(FROM_POPERTY, source.getAbsolutePath());
		properties.put(TO_PROPERTY, destination.getAbsolutePath());
		target = "q.move";
		return this;
	}

	@Flag(group = "main", values = { "$url" })
	@Doc("Delete a collection of files. One argument: directory to delete")
	public AntShellCommand delete(final File dir) {
		properties.put(FROM_POPERTY, dir.getAbsolutePath());
		target = "q.delete";
		return this;
	}

	@Flag(group = "main", values = { "`user.xml`", "`target`" })
	@Doc("Execute a target in a custom build file. Two arguments: 1) name of the build file 2) name of the target")
	public AntShellCommand other(final String file, final String target) {
		xmlFile = "ant/" + file;
		this.target = target;
		return this;
	}

	@Flag(values = "`**`")
	@Doc("Use only files matching the specified Ant wildcard. Use '?' to match a single character, '*' for multiple.")
	public AntShellCommand include(final String includes) {
		properties.put("q.includes", includes);
		return this;
	}

	@Flag(values = "``")
	@Doc("Exclude files matching the specified Ant wildcard. Use '?' to match a single character, '*' for multiple.")
	public AntShellCommand exclude(final String excludes) {
		properties.put("q.excludes", excludes);
		return this;
	}

	@Flag
	@Doc("Flatten source directory structure. File operation uses a single directory for the destination.")
	public AntShellCommand flatten() {
		properties.put("q.flatten", "true");
		return this;
	}

	@Flag(values = "`key1=value1#key2=value2`")
	@Doc("Specify custom properties to set. Properties are separated by '#'. Key / values are separated by '='")
	public AntShellCommand properties(final Map<String, String> map) {
		properties.putAll(map);
		return this;
	}

	@Flag(group = "main", values = { "$url", "$dir~", "`unpacked`" })
	@Doc("Unpack archives. 3 arguments: 1) archive to unpack 2) destination dir 3) put contents in nested directory")
	public AntShellCommand unzip(final File content, final File dir, final String nested) {
		target = UnpackTargets.getTarget(content);
		properties.put("q.archive", content.getAbsolutePath());
		properties.put("q.destination", new File(dir, nested).getAbsolutePath());
		return this;
	}

	@Flag
	@Doc("Use more verbose logging.")
	public AntShellCommand verbose() {
		properties.put("q.verbose", "true");
		return this;
	}

	@Flag(group = "main", values = { "$url", "$dir~", "$name" })
	@Doc("Create zip file. 3 arguments: 1) contents 2) destination dir 3) zip file name")
	public AntShellCommand zip(final File content, final File dir, final String nested) {
		target = "q.zip";
		String name = nested.endsWith(".zip") ? nested : nested + ".zip";
		properties.put("q.source", content.getAbsolutePath());
		properties.put("q.archive", new File(dir, name).getAbsolutePath());
		return this;
	}

	@Override
	public void validate() throws ValidationException {
	}

	@Override
	public void invoke() throws Exception {
		try {
			invokeBuildFile();
		} catch (IOException e) {
			throw new RuntimeIOException("Unable to find build file: %", xmlFile);
		}
		events.refresh();
	}

	private void invokeBuildFile() throws IOException {
		String url = classpath.getAbsolutePath(xmlFile);
		Map<String, String> props = MapTool.merge(plugin.getVariables(), properties);

		log.debug(String.format("Running Ant %s target %s", xmlFile, target));
		new AntProject(url, props).runTarget(target);
	}
}
