package org.q.plugins.execute;

import java.awt.Desktop;
import java.io.File;
import java.net.URI;

import org.q.app.config.ResourcePaths;
import org.q.app.contract.Invocable;
import org.q.app.contract.Swallows;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.app.ui.context.BrowseEvents;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.Str;
import org.q.toolkit.runtime.Command;
import org.q.toolkit.runtime.CommandInvoker;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.FORK)
@Doc("Execute files in system specific browsers, editors, etc.")
class ExecuteShellCommand implements Invocable, Swallows {

	private Mode mode = Mode.COMMAND;
	private File file;
	private URI uri;
	private String shell;
	private boolean refresh;
	private boolean frame = false;

	@Inject
	private BrowseEvents events;
	@Inject
	private SwingScript script;

	private enum Mode {
		BROWSE, OPEN, EDIT, COMMAND, TERMINAL
	}

	@Flag(group = "main", values = "$dir")
	@Doc("Open a URI in the system browser.")
	public ExecuteShellCommand browse(final URI uri) {
		mode = Mode.BROWSE;
		this.uri = uri;
		return this;
	}

	@Flag(group = "main", values = "$dir", primary = true)
	@Doc("Execute a command on the native command line.")
	public ExecuteShellCommand command(final File file) {
		mode = Mode.COMMAND;
		this.file = file;
		return this;
	}

	@Flag(group = "main", values = "$url")
	@Doc("Open a File with the system default application.")
	public ExecuteShellCommand open(final File file) {
		mode = Mode.OPEN;
		this.file = file;
		return this;
	}

	@Flag(group = "main", values = "$url")
	@Doc("Edit a File with the system default editor.")
	public ExecuteShellCommand edit(final File file) {
		mode = Mode.EDIT;
		this.file = file;
		return this;
	}

	@Flag(group = "main", values = "$dir")
	@Doc("Open a terminal window.")
	public ExecuteShellCommand terminal(final File file) {
		mode = Mode.TERMINAL;
		this.file = file;
		return this;
	}

	@Flag(group = "refresh")
	@Doc("Refresh the filebrowser after executing.")
	public ExecuteShellCommand refresh() {
		refresh = true;
		return this;
	}

	@Flag(group = "shell")
	@Doc("Invoke the native shell in order to be able to use features such as pipes '|' or redirects '>'.")
	public ExecuteShellCommand shell() {
		shell = isWindows() ? "cmd /c start cmd.exe" : "sh -c";
		return this;
	}

	@Flag(group = "output")
	@Doc("Show the resulting output.")
	public ExecuteShellCommand frame() {
		frame = true;
		return this;
	}

	@Override
	public void invoke() throws Exception {
		invoke(Str.EMPTY);
	}

	@Override
	@Doc("Command to invoke on the native command line.")
	public void invoke(final String raw) throws Exception {
		showTerminal();

		switch (mode) {
		case BROWSE:
			Desktop.getDesktop().browse(uri);
			return;

		case COMMAND:
			executeCommand(raw);
			return;

		case EDIT:
			Desktop.getDesktop().edit(file);
			return;

		case OPEN:
			Desktop.getDesktop().open(file);
			return;

		case TERMINAL:
			executeCommand(getTerminalCommand());
			return;

		default:
			throw new IllegalStateException("Unknown enum type: " + mode);
		}
	}

	private String getTerminalCommand() {
		return isWindows() ? "cmd" : "xterm";
	}

	private boolean isWindows() {
		return System.getProperty("os.name").toLowerCase().contains("windows");
	}

	private void showTerminal() {
		if (frame) {
			script.invoke(new ScriptInvocation(ResourcePaths.UI_TERMINAL_SHOW_SCRIPT));
		}
	}

	private void executeCommand(final String raw) {
		Command cmd = shell == null ? Command.from(raw) : nativeShellCommand(raw);
		CommandInvoker.invoke(cmd, file);
		if (refresh) {
			events.refresh();
		}
	}

	Command nativeShellCommand(final String raw) {
		String[] split = shell.split("\\s", 2);
		return new Command(split[0], split[1], raw);
	}

	@Override
	public void validate() throws ValidationException {
		boolean missing = file != null && !file.exists();
		RuntimeIOException.on(missing, "Execute failed, file does not exist: %", file.getAbsolutePath());
	}

	@Override
	public void validate(final String raw) throws ValidationException {
		validate();
		Command.from(raw);
	}
}
