package org.q.plugins.find;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.q.app.contract.SingleThreadInterruptedException;
import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFiles;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.pattern.WildcardTool;

class FileFinder extends Observable {
	static final int MAX_RESULTS = 500;
	static final LogTemplate LOG = new LogTemplate(FileFinder.class);

	private final LogTemplate log = new LogTemplate(getClass());
	private final List<VirtualFile> matches = new ArrayList<VirtualFile>();
	private VirtualFile dir;
	private String excludes;
	private String includes;
	private String contains;
	private Pattern pattern;
	private boolean onlyDirectories = false;
	private boolean onlyFiles = false;
	private boolean nested = true;
	private boolean path = false;

	FileFinder() {
	}

	FileFinder(final VirtualFile dir) {
		this.dir = dir;
	}

	void setDir(final VirtualFile dir) {
		this.dir = dir;
	}

	VirtualFile getDir() {
		return dir;
	}

	void setExcludes(final String excludes) {
		this.excludes = excludes.toLowerCase();
	}

	void setIncludes(final String includes) {
		this.includes = includes.toLowerCase();
	}

	void setRegex(final Pattern regex) {
		this.pattern = regex;
	}

	void onlyDirectories() {
		onlyDirectories = true;
	}

	void onlyFiles() {
		onlyFiles = true;
	}

	void setPathRegex() {
		path = true;
	}

	boolean isPathRegex() {
		return path;
	}

	void searchLocal() {
		nested = false;
	}

	private boolean isMaxResultsReached() {
		return matches.size() >= MAX_RESULTS;
	}

	public String getContains() {
		return contains;
	}

	public void setContains(final String contains) {
		this.contains = contains.toLowerCase();
	}

	List<VirtualFile> match() {
		for (VirtualFile file : VirtualFiles.getChildrenAlphabetically(dir)) {
			match(StringUtils.EMPTY, file);
		}
		return matches;
	}

	private void match(final String prefix, final VirtualFile file) {
		if (Thread.interrupted()) {
			throw new SingleThreadInterruptedException();
		}
		if (isMaxResultsReached()) {
			return;
		}
		addFileIfMatches(prefix, file);
		matchChildren(prefix, file);
	}

	private void matchChildren(final String prefix, final VirtualFile file) {
		try {
			if (nested && file.isDirectory()) {
				for (VirtualFile child : file.listFiles()) {
					match(appendPath(prefix, file), child);
				}
			}
		} catch (RuntimeIOException ioe) {
			log.warn("Unable to search child directories", ioe);
		}
	}

	private void addFileIfMatches(final String prefix, final VirtualFile file) {
		if (matchesTypes(file) && !matchesExcludes(file) && matchesIncludes(file)) {
			if (matchesRegex(getMatchPath(prefix, file)) && matchesContains(file)) {
				matches.add(file);
				setChanged();
				notifyObservers(file);
			}
		}
	}

	private boolean matchesContains(final VirtualFile file) {
		try {
			return contains == null ? true : VirtualFiles.toString(file).toLowerCase().contains(contains);

		} catch (RuntimeIOException e) {
			log.warn("Can't read file $", file);
			return false;
		}
	}

	private String appendPath(final String prefix, final VirtualFile child) {
		return StringUtils.isBlank(prefix) ? child.getName() : UrlTool.append(prefix, child.getName());
	}

	private String getMatchPath(final String prefix, final VirtualFile child) {
		return path ? UrlTool.append(prefix, child.getName()) : child.getName();
	}

	private boolean matchesExcludes(final VirtualFile file) {
		return excludes != null && WildcardTool.match(excludes, file.getName().toLowerCase());
	}

	private boolean matchesIncludes(final VirtualFile file) {
		return includes == null || WildcardTool.match(includes, file.getName().toLowerCase());
	}

	private boolean matchesRegex(final String input) {
		return pattern == null || pattern.matcher(input).matches();
	}

	private boolean matchesTypes(final VirtualFile file) {
		if (onlyDirectories) {
			return file.isDirectory();
		}
		if (onlyFiles) {
			return file.isFile();
		}
		return true;
	}
}
