package org.q.plugins.various;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFiles;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.MapTool;
import org.q.toolkit.lang.pattern.PatternTool;

class FileMatcher {

	private final VirtualFile filesParent;
	private final VirtualFile dirsParent;

	private String dirSelector = "(.*+)";
	private Pattern dirWords = Pattern.compile("\\s++");
	private String fileSelector = dirSelector;

	FileMatcher(final VirtualFile files, final VirtualFile dirs) {
		this.dirsParent = dirs;
		this.filesParent = files;

		Check.notNull(files, dirs);
		Check.isTrue(files.exists(), "File does not exist: %", files);
		Check.isTrue(dirs.exists(), "File does not exist: %", dirs);
	}

	void setFileSelector(final String regex) {
		checkSingleGroup(regex);
		fileSelector = regex;
	}

	void setDirSelector(final String regex) {
		checkSingleGroup(regex);
		dirSelector = regex;
	}

	void setDirWords(final String regex) {
		dirWords = Pattern.compile(regex);
	}

	Map<String, ?> createProperties() {
		return MapTool.of("from", filesParent, "to", dirsParent);
	}

	private void checkSingleGroup(final String regex) {
		boolean valid = Pattern.compile(regex).matcher("").groupCount() == 1;
		Validate.isTrue(valid, "Expected exactly 1 capturing group in regex: '" + regex + "'");
	}

	static String applySelector(final String regex, final String name) {
		return PatternTool.getGroup1(regex, name);
	}

	private List<String> getSearchWords(final String name) {
		return Arrays.asList(dirWords.split(name));
	}

	Map<VirtualFile, List<VirtualFile>> match() {
		List<VirtualFile> files = listFiles();

		Map<VirtualFile, List<VirtualFile>> result = new Hashtable<VirtualFile, List<VirtualFile>>();
		for (VirtualFile dir : listDirectories()) {
			List<String> search = getSearchWords(applySelector(dirSelector, dir.getBaseName()));
			List<VirtualFile> matches = findMatchingFiles(files, search);
			if (matches.size() > 0) {
				result.put(dir, matches);
			}
		}
		return result;
	}

	List<VirtualFile> findMatchingFiles(final List<VirtualFile> files, final List<String> search) {
		List<VirtualFile> matches = new ArrayList<VirtualFile>();
		for (VirtualFile file : files) {
			String name = applySelector(fileSelector, file.getBaseName());
			matches.add(file);
			for (String word : search) {
				if (!StringUtils.containsIgnoreCase(name, word)) {
					matches.remove(file);
					break;
				}
			}
		}
		return matches;
	}

	private List<VirtualFile> listFiles() {
		List<VirtualFile> result = new ArrayList<VirtualFile>();
		for (VirtualFile file : VirtualFiles.getChildrenAlphabetically(filesParent)) {
			if (file.isFile() && file.exists()) {
				result.add(file);
			}
		}
		return result;
	}

	private List<VirtualFile> listDirectories() {
		List<VirtualFile> result = new ArrayList<VirtualFile>();
		for (VirtualFile file : dirsParent.listFiles()) {
			if (file.isDirectory() && file.exists()) {
				result.add(file);
			}
		}
		return result;
	}

}
