package org.q.toolkit.io;

import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;

import org.q.toolkit.lang.LogTemplate;

/**
 * Utility class for accessing the OS clipboard
 * 
 * @author Erik Lievaart
 */
public class SystemClipboard {
	private final static LogTemplate log = new LogTemplate(SystemClipboard.class);

	private SystemClipboard() {
	}

	/**
	 * Write a simple String to the clipboard.
	 */
	public static void writeString(final String data) {
		writeTransferable(new StringSelection(data));
	}

	/**
	 * Write a Transferable Object to the clipboard.
	 */
	public static void writeTransferable(final Transferable data) {
		Toolkit.getDefaultToolkit().getSystemClipboard().setContents(data, null);
	}

	/**
	 * Read the contents of the clipboard as a String.
	 */
	public static String readString() {
		return (String) readFlavor(DataFlavor.stringFlavor);
	}

	/**
	 * Try to read the contents of the clipboard in the supplied flavor.
	 * 
	 * @param <E>
	 *            auto cast the resulting data to this type.
	 * @param flavor
	 *            Flavor to read from the clipboard.
	 * @return the contents of the clipboard, or null if no data is available in this flavor.
	 */
	@SuppressWarnings("unchecked")
	public static <E> E readFlavor(final DataFlavor flavor) {
		try {
			return (E) Toolkit.getDefaultToolkit().getSystemClipboard().getData(flavor);
		} catch (Exception e) {
			return null;
		}
	}

	/**
	 * List all available flavors for the data currently on the clipboard.
	 */
	public static DataFlavor[] getDataFlavors() {
		return Toolkit.getDefaultToolkit().getSystemClipboard().getAvailableDataFlavors();
	}

	/**
	 * Log all available flavors.
	 */
	public static void dumpFlavors() {
		for (DataFlavor flavor : getDataFlavors()) {
			log.info(flavor.toString());
		}
	}
}
