package org.q.toolkit.lang;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Utility class for writing log messages with arguments.
 * 
 * @author Erik Lievaart
 */
public class LogTemplate {

	private final Log log;

	/**
	 * Create a LogTemplate for the given name.
	 */
	public LogTemplate(final String name) {
		this.log = LogFactory.getLog(name);
	}

	/**
	 * Create a LogTemplate for the given literal.
	 */
	public LogTemplate(final Class<?> literal) {
		this.log = LogFactory.getLog(literal);
	}

	/**
	 * Is debug logging enabled flag.
	 */
	public boolean isDebugEnabled() {
		return log.isDebugEnabled();
	}

	/**
	 * Is trace logging enabled flag.
	 */
	public boolean isTraceEnabled() {
		return log.isTraceEnabled();
	}

	/**
	 * Write a trace level message (ignores formatting percentiles '%').
	 */
	public void trace(final String message) {
		log.trace(message);
	}

	/**
	 * Write a trace level message with the specified root cause (ignores formatting percentiles '%').
	 */
	public void trace(final String message, final Throwable t) {
		log.trace(message, t);
	}

	/**
	 * Write a formatted trace level message.
	 * 
	 * @see Str#sub(String, Object...)
	 */
	public void trace(final String format, final Object... args) {
		if (log.isTraceEnabled()) {
			log.trace(Str.sub(format, args));
		}
	}

	/**
	 * Write a debug level message (ignores formatting percentiles '%').
	 */
	public void debug(final String message) {
		log.debug(message);
	}

	/**
	 * Write a formatted debug level message.
	 * 
	 * @see Str#sub(String, Object...)
	 */
	public void debug(final String format, final Object... args) {
		if (log.isDebugEnabled()) {
			log.debug(Str.sub(format, args));
		}
	}

	/**
	 * Write a debug level message with the specified root cause (ignores formatting percentiles '%').
	 */
	public void debug(final String message, final Throwable e) {
		log.debug(message, e);
	}

	/**
	 * Write a info level message (ignores formatting percentiles '%').
	 */
	public void info(final String message) {
		log.info(message);
	}

	/**
	 * Write a formatted info level message.
	 * 
	 * @see Str#sub(String, Object...)
	 */
	public void info(final String format, final Object... args) {
		if (log.isInfoEnabled()) {
			log.info(Str.sub(format, args));
		}
	}

	/**
	 * Write a warn level message (ignores formatting percentiles '%').
	 */
	public void warn(final String message) {
		log.warn(message);
	}

	/**
	 * Write a formatted warn level message.
	 * 
	 * @see Str#sub(String, Object...)
	 */
	public void warn(final String format, final Object... args) {
		if (log.isWarnEnabled()) {
			log.warn(Str.sub(format, args));
		}
	}

	/**
	 * Write a warn level message with the specified root cause (ignores formatting percentiles '%').
	 */
	public void warn(final String message, final Throwable e) {
		log.warn(message, e);
	}

	/**
	 * Write a error level message with the specified root cause (ignores formatting percentiles '%').
	 */
	public void error(final String message, final Throwable e) {
		log.error(message, e);
	}

	/**
	 * Write a formatted error level message.
	 * 
	 * @see Str#sub(String, Object...)
	 */
	public void error(final String format, final Throwable e, Object... args) {
		log.error(Str.sub(format, args), e);
	}

	/**
	 * Get the underlying commons logging {@link Log}.
	 */
	public Log getLog() {
		return log;
	}

}
