package org.q.toolkit.lang.collection;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.q.toolkit.reflect.LiteralTool;

/**
 * Converts collections into other types.
 *
 * @author Erik Lievaart
 */
public class FromCollection {

	private FromCollection() {
	}

	/**
	 * Convert a {@link Collection} into a {@link Collection} of another type.
	 *
	 * @param <E>
	 *            type of the elements.
	 * @param type
	 *            Convert the collection to a collection of this type.
	 * @param original
	 *            the original collection.
	 * @return the elements in the original collection copied to a new collection of the specified type.
	 */
	public static <E> Collection<E> toType(final Class<?> type, final Collection<E> original) {
		Collection<E> collection = LiteralTool.newCollection(type);
		collection.addAll(original);
		return collection;
	}

	/**
	 * Place all the elements of the original collection in a List.
	 */
	public static <E> List<E> toList(final Collection<E> original) {
		return (List<E>) toType(List.class, original);
	}

	/**
	 * Place all the elements of the original enumeration in a List.
	 */
	public static <E> List<E> toList(final Enumeration<E> enumeration) {
		return Collections.list(enumeration);
	}

	/**
	 * Place all the elements of the original iterator in a List.
	 */
	public static <E> List<E> toList(final Iterator<E> iter) {
		List<E> list = NewCollection.list(NullPolicy.ACCEPT);
		while (iter.hasNext()) {
			list.add(iter.next());
		}
		return list;
	}

	/**
	 * Place all the elements in a read-write List.
	 */
	public static <E> List<E> toList(final E... args) {
		List<E> list = NewCollection.list(NullPolicy.ACCEPT);
		for (E item : args) {
			list.add(item);
		}
		return list;
	}

	/**
	 * Create an iterator containing the arguments.
	 *
	 * @param <E>
	 *            type of the elements.
	 */
	public static <E> Iterator<E> toIterator(final E... args) {
		return Arrays.asList(args).iterator();
	}

	/**
	 * Place all the elements of the original iterator in a Set.
	 */
	public static <E> Set<E> toSet(final Collection<E> collection) {
		return new HashSet<E>(collection);
	}
}
