package org.q.toolkit.lang.collection;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.q.toolkit.lang.Check;

/**
 * Utility class for working with Lists.
 * 
 * @author Erik Lievaart
 */
public class ListTool {

	private ListTool() {
	}

	/**
	 * Return a new List containing the elements from the specified index.
	 * 
	 * @param <E>
	 *            type of the elements.
	 * @param list
	 *            original List
	 * @param from
	 *            first index of the elements to return.
	 * @return the sublist.
	 */
	public static <E> List<E> subList(final List<E> list, final int from) {
		Check.notNull(list);
		Check.isTrue(list.size() >= from);
		return list.subList(from, list.size());
	}

	/**
	 * Return an unmodifiable List containing the elements in the original Collection.
	 */
	public static <E> List<E> unmodifiableCopy(final Collection<E> original) {
		return Collections.unmodifiableList(new ArrayList<E>(original));
	}

	/**
	 * Add all of the specified elements to the specified List.
	 */
	public static <E> void addAll(final List<E> list, final E[] elements) {
		list.addAll(Arrays.asList(elements));
	}

	/**
	 * Return a sorted copy of a list. Uses natural ordering.
	 */
	public static <E extends Comparable<? super E>> List<E> sortedCopy(final List<E> original) {
		List<E> copy = new ArrayList<E>(original);
		Collections.sort(copy);
		return copy;
	}

	/**
	 * Perform a binary search on a List with the specified key and return the first element equal to or greater than
	 * the key. The list must be sorted into ascending order according to the natural ordering of its elements.
	 * 
	 * @param <E>
	 *            Generic type of the key, the list and the result.
	 * @return Returns: the search key, if it is contained in the list; otherwise the insertion point. The insertion
	 *         point is defined as the the first element greater than the key, or null if all elements in the list are
	 *         less than the specified key.
	 */
	public static <E extends Comparable<? super E>> E lookup(final List<E> list, final E key) {
		int index = Collections.binarySearch(list, key);
		if (-index > list.size()) {
			return null;
		}
		return index < 0 ? list.get(-index - 1) : list.get(index);
	}

	/**
	 * Create a modifiable List from the specified elements.
	 */
	public static <E> List<E> of(final E... elements) {
		return new ArrayList<E>(Arrays.asList(elements));
	}
}
