package org.q.toolkit.runtime;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Invokes {@link Command} objects on the native command line.
 * 
 * @author Erik Lievaart
 */
public class CommandInvoker {
	private final static Log log = LogFactory.getLog(CommandInvoker.class);

	private static CommandOutput out = new CommandOutput() {
		@Override
		public void println(final String msg) {
			log.trace(msg);
		}
	};

	private static CommandOutput err = new CommandOutput() {
		@Override
		public void println(final String msg) {
			log.warn(msg);
		}
	};

	private CommandInvoker() {
	}

	/**
	 * Configure the system wide CommandOutput to write console messages to for any {@link Command} invoked through this
	 * {@link CommandInvoker}.
	 */
	public static void setNormalOutput(final CommandOutput output) {
		CommandInvoker.out = output;
	}

	/**
	 * Configure the system wide CommandOutput to write error messages to for any {@link Command} invoked through this
	 * {@link CommandInvoker}.
	 */
	public static void setErrorOutput(final CommandOutput output) {
		CommandInvoker.err = output;
	}

	/**
	 * Invokes the supplied {@link Command}.
	 */
	public static boolean invoke(final Command command) {
		return invoke(command, null);
	}

	/**
	 * Invokes the supplied {@link Command} in the specified directory.
	 */
	public static boolean invoke(final Command command, final File dir) {
		try {
			int exit = execute(command, dir);
			out.println("command exited with code: " + exit);
			return exit == 0;

		} catch (IOException ioe) {
			err.println(ioe.getMessage());
			return false;
		}
	}

	private static int execute(final Command command, final File dir) throws IOException {
		assert command != null : "Cannot execute <null>";
		log.trace(command);
		Process process = Runtime.getRuntime().exec(command.cmd(), null, dir);

		// Reading the streams from the Process prevents the task from hanging.
		dumpInNewThread(process.getErrorStream(), err);
		dumpInCurrentThread(process.getInputStream(), out);
		try {
			return process.waitFor();
		} catch (InterruptedException e) {
			throw new IOException(e);
		}
	}

	/**
	 * Read the stream in a new Thread.
	 */
	private static void dumpInNewThread(final InputStream input, final CommandOutput out) {
		new Thread(new StreamRunnable(input, out, err)).start();
	}

	/**
	 * Read the stream in the currently active Thread.
	 */
	private static void dumpInCurrentThread(final InputStream input, final CommandOutput out) {
		new StreamRunnable(input, out, err).run();
	}

}
