package org.q.toolkit.swing;

import java.awt.AWTEvent;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.swing.JFrame;

import org.q.toolkit.io.ApplicationPaths;
import org.q.toolkit.io.PropertiesIO;
import org.q.toolkit.io.UrlTool;

/**
 * Utility class for saving and restoring the positions of all Window's used in the application. Simply call the
 * {@link WindowSaver#initialize()} method and all Window positions will be saved and restored automatically. Be sure to
 * call the setName() method on any window before making them visible.
 *
 * @author Erik Lievaart
 */
public class WindowSaver implements AWTEventListener {
	private static final WindowSaver INSTANCE = new WindowSaver();
	private static final String PROPERTIES_FILE = UrlTool.append(ApplicationPaths.getJarDirOrRunDir(WindowSaver.class),
			"data/WindowSaver.ini");

	private final Map<String, JFrame> framemap = new HashMap<String, JFrame>();

	private WindowSaver() {
	}

	private static WindowSaver instance() {
		return INSTANCE;
	}

	/**
	 * Save and restore all Window positions automatically.
	 */
	public static void initialize() {
		Toolkit tk = Toolkit.getDefaultToolkit();
		tk.addAWTEventListener(instance(), AWTEvent.WINDOW_EVENT_MASK);
	}

	@Override
	public void eventDispatched(final AWTEvent evt) {
		if (evt.getID() == WindowEvent.WINDOW_OPENED) {
			ComponentEvent cev = (ComponentEvent) evt;
			if (cev.getComponent() instanceof JFrame) {
				loadSettings((JFrame) cev.getComponent());
			}
		}
		if (evt.getID() == WindowEvent.WINDOW_CLOSED) {
			saveSettings();
		}
		if (evt.getID() == WindowEvent.WINDOW_CLOSING) {
			saveSettings();
		}
	}

	private static void loadSettings(final JFrame frame) {
		Properties settings = loadProperties();
		String name = frame.getName();
		checkFrameName(name);

		int x = getInt(settings, name + ".x", 100);
		int y = getInt(settings, name + ".y", 100);
		int w = getInt(settings, name + ".w", 440);
		int h = getInt(settings, name + ".h", 440);
		frame.setBounds(x, y, w, h);
		INSTANCE.framemap.put(name, frame);
		frame.validate();
	}

	private static void checkFrameName(final String name) {
		if (name.matches("frame\\d")) {
			throw new IllegalArgumentException("Please call setName(String) on all JFrames");
		}
	}

	private static Properties loadProperties() {
		try {
			return PropertiesIO.load(new File(PROPERTIES_FILE));
		} catch (IOException ioe) {
			return new Properties();
		}
	}

	private static int getInt(final Properties props, final String name, final int def) {
		String value = props.getProperty(name);
		if (value == null) {
			return def;
		}
		return Integer.parseInt(value);
	}

	private static void saveSettings() {
		try {
			PropertiesIO.store(prepareData(), new File(PROPERTIES_FILE));
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	private static Properties prepareData() {
		Properties settings = loadProperties();

		for (String name : INSTANCE.framemap.keySet()) {
			checkFrameName(name);

			JFrame frame = INSTANCE.framemap.get(name);
			settings.setProperty(name + ".x", "" + frame.getX());
			settings.setProperty(name + ".y", "" + frame.getY());
			settings.setProperty(name + ".w", "" + frame.getWidth());
			settings.setProperty(name + ".h", "" + frame.getHeight());
		}
		return settings;
	}
}
